#
# flex.py
#
# receive and display temperature
# hello.temp.45.py serial_port
#
# Neil Gershenfeld
# CBA MIT 3/27/12
#
# (c) Massachusetts Institute of Technology 2012
# Permission granted for experimental and personal use;
# license for commercial sale available from MIT
#
# Modified by Victoria Peredo
# FabLab Madrid CEU

from Tkinter import *
from numpy import log
import serial

WINDOW = 600 # window size
eps = 0.5 # filter time constant
filter = 0.0 # filtered value
vcc = 2.98
div_resis = 24600 #Measured resistance value of the voltage divider res

def idle(parent,canvas):
   global filter, eps
   #
   # idle routine
   #
   byte2 = 0
   byte3 = 0
   byte4 = 0
   ser.flush()
   while 1:
      #
      # find framing 
      #
      byte1 = byte2
      byte2 = byte3
      byte3 = byte4
      byte4 = ord(ser.read())
      if ((byte1 == 1) & (byte2 == 2) & (byte3 == 3) & (byte4 == 4)):
         break
   low = ord(ser.read())
   high = ord(ser.read())
   adc = 256*high + low
   flexVolt = adc*vcc/1024
   #flexRes = div_resis*(vcc/flexVolt-1)
   flexRes = (((vcc*div_resis)/flexVolt)-div_resis)/1000

   #x = int(.2*WINDOW + (.9-.2)*WINDOW*flexRes)
   canvas.itemconfigure("adc",text="ADC: %.1f"%adc)
   canvas.itemconfigure("volt",text="Volt of res: %.1f"%flexVolt)
   canvas.itemconfigure("res",text="Res (Kohms): %.1f"%flexRes)
   #canvas.coords('rect1',.2*WINDOW,.05*WINDOW,x,.2*WINDOW)
   #canvas.coords('rect2',x,.05*WINDOW,.9*WINDOW,.2*WINDOW)
   canvas.update()
   parent.after_idle(idle,parent,canvas)


#
#  check command line arguments
#
if (len(sys.argv) != 2):
   print "command line: hello.temp.45.py serial_port"
   sys.exit()
port = sys.argv[1]
#
# open serial port
#
ser = serial.Serial(port,9600)
ser.setDTR()
#
# start plotting
#
root = Tk()
root.title('Thread resistance (q to exit)')
root.bind('q','exit')
canvas = Canvas(root, width=WINDOW, height=WINDOW, background='#eeeded')
canvas.create_text(.5*WINDOW,.1*WINDOW,text=".33",font=("Helvetica", 40),tags="adc",fill="#000000")
canvas.create_text(.5*WINDOW,.4*WINDOW,text=".33",font=("Helvetica", 40),tags="volt",fill="#000000")
canvas.create_text(.5*WINDOW,.7*WINDOW,text=".33",font=("Helvetica", 40),tags="res",fill="#000000")
#canvas.create_rectangle(.2*WINDOW,.05*WINDOW,.3*WINDOW,.2*WINDOW, tags='rect1', fill='#62d1ff')
#canvas.create_rectangle(.3*WINDOW,.05*WINDOW,.9*WINDOW,.2*WINDOW, tags='rect2', fill='#ffffff')
canvas.pack()
root.after(100,idle,root,canvas)
root.mainloop()
